from .grpc_lib.blog_pb2 import (
    BlogListRequest,
    BlogRetrieveRequest,
    CategoryRetrieveRequest,
)
from .grpc_lib.blog_pb2_grpc import BlogControllerStub, CategoryControllerStub

SUCCESS_CODE = "0"


class RequestMessages:
    FETCH_ALL_BLOG = BlogListRequest
    FETCH_BLOG = BlogRetrieveRequest
    FETCH_CATEGORY = CategoryRetrieveRequest


class RPCMethodNames:
    FETCH_ONE = "Retrieve"
    FETCH_ALL = "List"


class ControllerStubs:
    BLOG = BlogControllerStub()
    CATEGORY = CategoryControllerStub()


class InvokerUnits:
    PLACETAG_SITEMAP = "placetag_sitemap"
    SEARCH = "search"



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""

import grpc

from . import blog_pb2 as blog_dot_grpc_dot_blog__pb2


class BlogControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.List = self.channel.unary_unary(
                "/config.blog.BlogController/List",
                request_serializer=blog_dot_grpc_dot_blog__pb2.BlogListRequest.SerializeToString,
                response_deserializer=blog_dot_grpc_dot_blog__pb2.BlogResponse.FromString,
            )
            self.Retrieve = self.channel.unary_unary(
                "/config.blog.BlogController/Retrieve",
                request_serializer=blog_dot_grpc_dot_blog__pb2.BlogRetrieveRequest.SerializeToString,
                response_deserializer=blog_dot_grpc_dot_blog__pb2.BlogResponse.FromString,
            )


class BlogControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def List(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_BlogControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "List": grpc.unary_unary_rpc_method_handler(
            servicer.List,
            request_deserializer=blog_dot_grpc_dot_blog__pb2.BlogListRequest.FromString,
            response_serializer=blog_dot_grpc_dot_blog__pb2.BlogResponse.SerializeToString,
        ),
        "Retrieve": grpc.unary_unary_rpc_method_handler(
            servicer.Retrieve,
            request_deserializer=blog_dot_grpc_dot_blog__pb2.BlogRetrieveRequest.FromString,
            response_serializer=blog_dot_grpc_dot_blog__pb2.BlogResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.blog.BlogController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class BlogController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def List(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.blog.BlogController/List",
            blog_dot_grpc_dot_blog__pb2.BlogListRequest.SerializeToString,
            blog_dot_grpc_dot_blog__pb2.BlogResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def Retrieve(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.blog.BlogController/Retrieve",
            blog_dot_grpc_dot_blog__pb2.BlogRetrieveRequest.SerializeToString,
            blog_dot_grpc_dot_blog__pb2.BlogResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class CategoryControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.Retrieve = self.channel.unary_unary(
                "/config.blog.CategoryController/Retrieve",
                request_serializer=blog_dot_grpc_dot_blog__pb2.CategoryRetrieveRequest.SerializeToString,
                response_deserializer=blog_dot_grpc_dot_blog__pb2.CategoryResponse.FromString,
            )


class CategoryControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_CategoryControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Retrieve": grpc.unary_unary_rpc_method_handler(
            servicer.Retrieve,
            request_deserializer=blog_dot_grpc_dot_blog__pb2.CategoryRetrieveRequest.FromString,
            response_serializer=blog_dot_grpc_dot_blog__pb2.CategoryResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.blog.CategoryController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class CategoryController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Retrieve(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.blog.CategoryController/Retrieve",
            blog_dot_grpc_dot_blog__pb2.CategoryRetrieveRequest.SerializeToString,
            blog_dot_grpc_dot_blog__pb2.CategoryResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



from typing import ClassVar, Optional

from google.protobuf.json_format import MessageToDict
from grpc.aio import insecure_channel
from pydantic import UUID4, BaseModel, validate_call

from .api_constants import (
    SUCCESS_CODE,
    ControllerStubs,
    InvokerUnits,
    RequestMessages,
    RPCMethodNames,
)


class ValidationError(Exception):
    pass


class ResponseCodeError(Exception):
    pass


class RPCClient(BaseModel):
    """
    Configure the host and port of the search server.

    Parameters:
        host (str): The host address of the search server.
        port (int): The port number of the search server.
    """

    host: str = "127.0.0.1"
    port: str | int = "50051"

    _instances: ClassVar[dict] = {}

    def __new__(cls, *args, **kwargs):
        # Ensure that each subclass has its own singleton instance
        if cls not in cls._instances:
            cls._instances[cls] = super().__new__(cls)
        return cls._instances[cls]

    async def perform_operation(self, rpc_method_name, request_message=None):
        """Perform the operation.

        Parameters:
            rpc_method_name (str): The name of the RPC method (e.g., `Comment`).
            request_message (gRPC message): The request message instance.
        """
        async with insecure_channel(f"{self.host}:{self.port}") as channel:
            self.controller_stub.channel = channel
            self.controller_stub.ready()
            RPCMethod = getattr(self.controller_stub, rpc_method_name, None)

            if RPCMethod:
                response = await RPCMethod(request_message)

                # Clear the request message for future use if is not None
                if request_message is not None:
                    request_message.Clear()

                # Raise error if response is not successful
                if response.code != SUCCESS_CODE:
                    raise ResponseCodeError(response.code)

                return MessageToDict(response, preserving_proto_field_name=True)
            else:
                raise ValidationError("Invalid RPC method name.")


class RPCBase(RPCClient):
    """Base class for RPC operations, extending RPCClient for handling RPC calls."""

    @validate_call
    async def fetch_one(
        self, identifier: Optional[str | UUID4] = None, slug: Optional[str] = None
    ):
        """Fetch a single record from a database model."""
        request = self.fetch_one_request()
        if identifier:
            request.identifier = identifier  # Assign attributes
        if slug:
            request.slug = slug  # Assign attributes

        return await self.perform_operation(RPCMethodNames.FETCH_ONE, request)

    @validate_call
    async def fetch_one_for_search(
        self, identifier: Optional[str | UUID4] = None, slug: Optional[str] = None
    ):
        """Fetch a single record from a database model with invoker_unit set to `search`."""
        request = self.fetch_one_request(invoker_unit=InvokerUnits.SEARCH)
        if identifier:
            request.identifier = identifier  # Assign attributes
        if slug:
            request.slug = slug  # Assign attributes

        return await self.perform_operation(RPCMethodNames.FETCH_ONE, request)

    @validate_call
    async def fetch_all(self, order_by: str, page_size: int, page_number: int):
        """Fetch all records from a database model."""
        request = self.fetch_all_request(
            page_size=page_size, order_by=order_by, page_number=page_number
        )
        return await self.perform_operation(RPCMethodNames.FETCH_ALL, request)

    @validate_call
    async def fetch_all_for_sitemap(
        self, order_by: str, page_size: int, page_number: int
    ):
        """Fetch all records from a database model for sitemap use."""
        request = self.fetch_all_request(
            page_size=page_size,
            order_by=order_by,
            page_number=page_number,
            invoker_unit=InvokerUnits.PLACETAG_SITEMAP,
        )
        return await self.perform_operation(RPCMethodNames.FETCH_ALL, request)


class Blog(RPCBase):
    """`fetch_one()`, `featch_all()`."""

    controller_stub: ClassVar = ControllerStubs.BLOG
    fetch_one_request: ClassVar = RequestMessages.FETCH_BLOG
    fetch_all_request: ClassVar = RequestMessages.FETCH_ALL_BLOG


class Category(RPCClient):
    """`fetch_one()`."""

    controller_stub: ClassVar = ControllerStubs.CATEGORY
    fetch_one_request: ClassVar = RequestMessages.FETCH_CATEGORY

    @validate_call
    async def fetch_one(
        self, identifier: Optional[str | UUID4] = None, slug: Optional[str] = None
    ):
        """Fetch category data."""
        request = self.fetch_one_request()
        if identifier:
            request.identifier = identifier  # Assign attributes
        if slug:
            request.slug = slug  # Assign attributes

        return await self.perform_operation(RPCMethodNames.FETCH_ONE, request)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blog/grpc/blog.proto
# Protobuf Python Version: 4.25.1
# pylint: skip-file
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x14\x62log/grpc/blog.proto\x12\x0b\x63onfig.blog\x1a\x1cgoogle/protobuf/struct.proto"s\n\x0f\x42logListRequest\x12\x15\n\x08order_by\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x0bpage_number\x18\x02 \x01(\x05\x12\x11\n\tpage_size\x18\x03 \x01(\x05\x12\x14\n\x0cinvoker_unit\x18\x04 \x01(\tB\x0b\n\t_order_by"T\n\x0c\x42logResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct"c\n\x13\x42logRetrieveRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x0c\n\x04slug\x18\x02 \x01(\t\x12\x19\n\x0cinvoker_unit\x18\x03 \x01(\tH\x00\x88\x01\x01\x42\x0f\n\r_invoker_unit"X\n\x10\x43\x61tegoryResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct"g\n\x17\x43\x61tegoryRetrieveRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x0c\n\x04slug\x18\x02 \x01(\t\x12\x19\n\x0cinvoker_unit\x18\x03 \x01(\tH\x00\x88\x01\x01\x42\x0f\n\r_invoker_unit2\x9e\x01\n\x0e\x42logController\x12\x41\n\x04List\x12\x1c.config.blog.BlogListRequest\x1a\x19.config.blog.BlogResponse"\x00\x12I\n\x08Retrieve\x12 .config.blog.BlogRetrieveRequest\x1a\x19.config.blog.BlogResponse"\x00\x32g\n\x12\x43\x61tegoryController\x12Q\n\x08Retrieve\x12$.config.blog.CategoryRetrieveRequest\x1a\x1d.config.blog.CategoryResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "blog.grpc.blog_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_BLOGLISTREQUEST"]._serialized_start = 67
    _globals["_BLOGLISTREQUEST"]._serialized_end = 182
    _globals["_BLOGRESPONSE"]._serialized_start = 184
    _globals["_BLOGRESPONSE"]._serialized_end = 268
    _globals["_BLOGRETRIEVEREQUEST"]._serialized_start = 270
    _globals["_BLOGRETRIEVEREQUEST"]._serialized_end = 369
    _globals["_CATEGORYRESPONSE"]._serialized_start = 371
    _globals["_CATEGORYRESPONSE"]._serialized_end = 459
    _globals["_CATEGORYRETRIEVEREQUEST"]._serialized_start = 461
    _globals["_CATEGORYRETRIEVEREQUEST"]._serialized_end = 564
    _globals["_BLOGCONTROLLER"]._serialized_start = 567
    _globals["_BLOGCONTROLLER"]._serialized_end = 725
    _globals["_CATEGORYCONTROLLER"]._serialized_start = 727
    _globals["_CATEGORYCONTROLLER"]._serialized_end = 830
# @@protoc_insertion_point(module_scope)
